==> odoo_18.1alpha1.20241219.zip: odoo-18.1a1.post20241219/odoo/addons/web/static/tests/_framework/async_step.js <==
import { after, expect } from "@odoo/hoot";
import { Deferred } from "@odoo/hoot-dom";

/**
 * @typedef {{
 *  steps: any[];
 *  expectedSteps: any[] | null;
 *  deferred: Deferred | null;
 *  timeout: number;
 * }} StepState
 *
 * @typedef {{
 *  timeout?: number;
 * }} WaitForStepsOptions
 */

//-----------------------------------------------------------------------------
// Internals
//-----------------------------------------------------------------------------

/**
 * @param {boolean} forceVerifySteps
 */
const checkStepState = (forceVerifySteps) => {
    if (!currentStepState || !currentStepState.expectedSteps) {
        return;
    }

    const { expectedSteps, steps } = currentStepState;
    if (
        forceVerifySteps ||
        (expectedSteps.length === steps.length && expectedSteps.every((s, i) => s === steps[i]))
    ) {
        expect.verifySteps(expectedSteps);
        clearStepState();
    }
};

const clearStepState = () => {
    if (!currentStepState) {
        return;
    }
    if (currentStepState.timeout) {
        clearTimeout(currentStepState.timeout);
    }
    // Never reject since `verifySteps` will already log an error if steps do not match
    currentStepState.deferred?.resolve();
    currentStepState = null;
};

const ensureStepState = () => {
    if (!currentStepState) {
        currentStepState = {
            steps: [],
            deferred: null,
            expectedSteps: null,
            timeout: 0,
        };
        after(runLastCheck);
    }
    return currentStepState;
};

const runLastCheck = () => {
    if (currentStepState?.steps.length) {
        checkStepState(true);
    } else {
        clearStepState();
    }
};

/** @type {StepState | null} */
let currentStepState = null;

//-----------------------------------------------------------------------------
// Exports
//-----------------------------------------------------------------------------

/**
 * Indicate the completion of a test step. This step must then be verified by
 * calling {@link waitForSteps}.
 *
 * @param {any} step
 */
export function asyncStep(step) {
    const stepState = ensureStepState();
    stepState.steps.push(step);

    expect.step(step);

    // Soft check with newly added step
    checkStepState(false);
}

/**
 * Wait for the given steps to be executed (by {@link asyncStep}), before
 * the end of a given timeout (default: 2000ms).
 *
 * @param {any[]} steps
 * @param {WaitForStepsOptions} [options]
 */
export async function waitForSteps(steps, options) {
    // Check with previous steps (if any)
    checkStepState(true);

    const stepState = ensureStepState();
    stepState.expectedSteps = steps;
    stepState.deferred = new Deferred();
    stepState.timeout = setTimeout(() => checkStepState(true), options?.timeout ?? 2000);

    // Soft check with current steps
    checkStepState(false);

    return stepState.deferred;
}

==> odoo_18.2alpha1.20250125.zip: odoo-18.2a1.post20250125/odoo/addons/web/static/tests/_framework/async_step.js <==
import { after, expect } from "@odoo/hoot";
import { Deferred } from "@odoo/hoot-dom";

/**
 * @typedef {{
 *  steps: any[];
 *  expectedSteps: any[] | null;
 *  deferred: Deferred | null;
 *  timeout: number;
 * }} StepState
 *
 * @typedef {{
 *  timeout?: number;
 *  ignoreOrder?: boolean;
 * }} WaitForStepsOptions
 */

//-----------------------------------------------------------------------------
// Internals
//-----------------------------------------------------------------------------

/**
 * @param {boolean} forceVerifySteps
 */
const checkStepState = (forceVerifySteps) => {
    if (!currentStepState || !currentStepState.expectedSteps) {
        return;
    }

    const { expectedSteps, ignoreOrder, steps } = currentStepState;
    if (ignoreOrder) {
        expectedSteps.sort(function (a, b) {
            return steps.indexOf(a) - steps.indexOf(b);
        });
    }
    if (
        forceVerifySteps ||
        (expectedSteps.length === steps.length && expectedSteps.every((s, i) => s === steps[i]))
    ) {
        expect.verifySteps(expectedSteps);
        clearStepState();
    }
};

const clearStepState = () => {
    if (!currentStepState) {
        return;
    }
    if (currentStepState.timeout) {
        clearTimeout(currentStepState.timeout);
    }
    // Never reject since `verifySteps` will already log an error if steps do not match
    currentStepState.deferred?.resolve();
    currentStepState = null;
};

const ensureStepState = () => {
    if (!currentStepState) {
        currentStepState = {
            steps: [],
            deferred: null,
            expectedSteps: null,
            ignoreOrder: false,
            timeout: 0,
        };
        after(runLastCheck);
    }
    return currentStepState;
};

const runLastCheck = () => {
    if (currentStepState?.steps.length) {
        checkStepState(true);
    } else {
        clearStepState();
    }
};

/** @type {StepState | null} */
let currentStepState = null;

//-----------------------------------------------------------------------------
// Exports
//-----------------------------------------------------------------------------

/**
 * Indicate the completion of a test step. This step must then be verified by
 * calling {@link waitForSteps}.
 *
 * @param {any} step
 */
export function asyncStep(step) {
    const stepState = ensureStepState();
    stepState.steps.push(step);

    expect.step(step);

    // Soft check with newly added step
    checkStepState(false);
}

/**
 * Wait for the given steps to be executed (by {@link asyncStep}), before
 * the end of a given timeout (default: 2000ms).
 *
 * @param {any[]} steps
 * @param {WaitForStepsOptions} [options]
 */
export async function waitForSteps(steps, options = {}) {
    // Check with previous steps (if any)
    checkStepState(true);

    const stepState = ensureStepState();
    stepState.expectedSteps = steps;
    stepState.deferred = new Deferred();
    stepState.ignoreOrder = options.ignoreOrder;
    stepState.timeout = setTimeout(() => checkStepState(true), options.timeout ?? 2000);

    // Soft check with current steps
    checkStepState(false);

    return stepState.deferred;
}
